@@ -1,17 +1,17 @@
-""" Perturb regions in bedfiles """
+"""Perturb regions in bedfiles"""
 
 import logging
-import logmuse
 import math
-import numpy as np
 import os
-import pandas as pd
 import random
 import sys
 
+import logmuse
+import numpy as np
+import pandas as pd
+
+from geniml.bedshift import BedshiftYAMLHandler, arguments
 from geniml.bedshift._version import __version__
-from geniml.bedshift import arguments
-from geniml.bedshift import BedshiftYAMLHandler
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -443,6 +443,8 @@ class Bedshift(object):
         raise NotImplementedError(
             "This relies on pyranges, which was removed with the switch to geniml."
         )
+
+        # USE AILIST HERE:
         reference_pr = pr.PyRanges(reference_bed)
         comparison_pr = pr.PyRanges(comparison_bed)
         intersection = reference_pr.overlap(comparison_pr, how="first").as_df()
