@@ -1,14 +1,17 @@
 import gzip
+import logging
+import warnings
+from typing_extensions import deprecated
 import os
-from typing import List, Union, NoReturn
-import pyarrow
-from ubiquerg import is_url
+from hashlib import md5
+from typing import List, NoReturn, Union
 
+import genomicranges
 import numpy as np
 import pandas as pd
-import genomicranges
 from iranges import IRanges
-from hashlib import md5
+from ubiquerg import is_url
+from gtars.models import RegionSet as GRegionSet
 
 from .const import (
     MAF_CENTER_COL_NAME,
@@ -21,10 +24,15 @@ from .const import (
     MAF_START_COL_NAME,
     MAF_STRAND_COL_NAME,
 )
-from .utils import extract_maf_col_positions, is_gzipped, read_bedset_file
-from .exceptions import BackedFileNotAvailableError
+from .exceptions import BackedFileNotAvailableError, BEDFileReadError
+from .utils import compute_md5sum_bedset, extract_maf_col_positions, is_gzipped, read_bedset_file
+
+_LOGGER = logging.getLogger("bbclient")
 
 
+@deprecated(
+    "This class is deprecated and will be removed in future versions. Use Region class from Gtars"
+)
 class Region:
     def __init__(self, chr: str, start: int, stop: int):
         """
@@ -34,6 +42,10 @@ class Region:
         :param int start: start position
         :param int stop: stop position
         """
+        warnings.warn(
+            "Region is deprecated and will be removed in future versions.", DeprecationWarning
+        )
+
         self.chr = chr
         self.start = start
         self.end = stop
@@ -42,6 +54,7 @@ class Region:
         return f"Region({self.chr}, {self.start}, {self.end})"
 
 
+@deprecated("This class is deprecated and will be removed in future versions.")
 class RegionSet:
     def __init__(self, regions: Union[str, List[Region]], backed: bool = False):
         """
@@ -51,14 +64,19 @@ class RegionSet:
         bed files. You can still iterate over the regions, but you cannot index into them.
 
         :param regions: path, or url to bed file or list of Region objects
-        :param backed: whether to load the bed file into memory or not
+        :param backed: whether to load the bed file into memory or not [Default: False]
         """
-        # load from file
+        warnings.warn(
+            "RegionSet is deprecated and will be removed in future versions. Use RegionSet class from Gtars",
+            DeprecationWarning,
+        )
+
+        self._df: Union[pd.DataFrame, None] = None
+
         if isinstance(regions, str):
             self.backed = backed
             self.regions: List[Region] = []
             self.path = regions
-
             self.regions = None
             self.is_gzipped = False
 
@@ -86,13 +104,14 @@ class RegionSet:
             else:
                 if is_gzipped(regions):
                     df = self._read_gzipped_file(regions)
-
                 else:
-                    # if file is gzipped, catch error and open using gzip
-                    try:
-                        df = pd.read_csv(regions, sep="\t", header=None, engine="pyarrow")
-                    except pyarrow.lib.ArrowInvalid:
-                        df = self._read_gzipped_file(regions)
+                    df = self._read_file_pd(
+                        regions,
+                        sep="\t",
+                        header=None,
+                        engine="pyarrow",
+                    )
+                self._df = df
 
                 _regions = []
                 df.apply(
@@ -114,22 +133,72 @@ class RegionSet:
 
         self._identifier = None
 
-    @staticmethod
-    def _read_gzipped_file(file_path: str) -> pd.DataFrame:
+    def to_pandas(self) -> Union[pd.DataFrame, None]:
+        if self._df is None:
+            seqnames, starts, ends = zip(
+                *[(region.chr, region.start, region.end) for region in self]
+            )
+            return pd.DataFrame([seqnames, starts, ends])
+
+        return self._df
+
+    def _read_gzipped_file(self, file_path: str) -> pd.DataFrame:
         """
         Read a gzipped file into a pandas dataframe
 
         :param file_path: path to gzipped file
         :return: pandas dataframe
         """
-        return pd.read_csv(
+        return self._read_file_pd(
             file_path,
             sep="\t",
             compression="gzip",
             header=None,
-            engine="pyarrow",
+            # engine="pyarrow",
         )
 
+    def _read_file_pd(self, *args, **kwargs) -> pd.DataFrame:
+        """
+        Read bed file into a pandas DataFrame, and skip header rows if needed
+
+        :return: pandas dataframe
+        """
+        max_rows = 5
+        row_count = 0
+        while row_count <= max_rows:
+            try:
+                df = pd.read_csv(*args, **kwargs, skiprows=row_count)
+                if row_count > 0:
+                    _LOGGER.info(f"Skipped {row_count} rows while standardization. File: '{args}'")
+                df = df.dropna(axis=1)
+                for index, row in df.iterrows():
+                    if (
+                        isinstance(row[0], str)
+                        and isinstance(row[1], int)
+                        and isinstance(row[2], int)
+                    ):
+                        return df
+                    else:
+                        if isinstance(row[1], str):
+                            try:
+                                _ = int(row[1])
+                                df[1] = pd.to_numeric(df[1])
+                            except ValueError:
+                                row_count += 1
+                                break
+                        if isinstance(row[2], str):
+                            try:
+                                _ = int(row[2])
+                                df[2] = pd.to_numeric(df[2])
+                            except ValueError:
+                                row_count += 1
+                                break
+                        return df
+            except (pd.errors.ParserError, pd.errors.EmptyDataError) as _:
+                if row_count <= max_rows:
+                    row_count += 1
+        raise BEDFileReadError("Cannot read bed file.")
+
     def __len__(self):
         return self.length
 
@@ -159,8 +228,21 @@ class RegionSet:
                 mode = "r"
 
             with open_func(self.path, mode) as f:
+                skipped_lines = 0
+                max_skipped_lines = 5
                 for line in f:
-                    chr, start, stop = line.split("\t")[:3]
+                    try:
+                        chr, start, stop = line.split("\t")[:3]
+                    except ValueError as _:
+                        if skipped_lines < max_skipped_lines:
+                            skipped_lines += 1
+                            continue
+                        else:
+                            raise BEDFileReadError("Could not read line bed file")
+                    if skipped_lines > 0:
+                        _LOGGER.info(
+                            f"Skipped {skipped_lines} lines while opening file. File: '{self.path}'"
+                        )
                     yield Region(chr, int(start), int(stop))
         else:
             for region in self.regions:
@@ -198,7 +280,7 @@ class RegionSet:
         else:
             if not self.backed:
                 # concate column values
-                chrs = ",".join([region.chr for region in self.regions])
+                chrs = ",".join([str(region.chr) for region in self.regions])
                 starts = ",".join([str(region.start) for region in self.regions])
                 ends = ",".join([str(region.end) for region in self.regions])
 
@@ -257,11 +339,11 @@ class BedSet:
             else:
                 self.region_sets = []
                 for r in region_sets:
-                    self.region_sets.append(RegionSet(r))
+                    self.region_sets.append(GRegionSet(r))
 
         elif file_path is not None:
             if os.path.isfile(file_path):
-                self.region_sets = [RegionSet(r) for r in read_bedset_file(file_path)]
+                self.region_sets = [GRegionSet(r) for r in read_bedset_file(file_path)]
             else:
                 raise FileNotFoundError(f"The specified file '{file_path}' does not exist.")
         else:
@@ -320,10 +402,8 @@ class BedSet:
         elif self._bedset_identifier is None:
             bedfile_ids = []
             for bedfile in self.region_sets:
-                bedfile_ids.append(bedfile.compute_bed_identifier())
-            self._bedset_identifier = md5(
-                ";".join(sorted(bedfile_ids)).encode("utf-8")
-            ).hexdigest()
+                bedfile_ids.append(bedfile.identifier)
+            self._bedset_identifier = compute_md5sum_bedset(bedfile_ids)
 
             return self._bedset_identifier
 
