@@ -1,18 +1,38 @@
 import logging
 import math
 import os
-from typing import Dict, List, Tuple, Union
+from typing import List, Tuple, Union
 
 import matplotlib.pyplot as plt
 import numpy as np
-import torch
-from fastembed.embedding import FlagEmbedding
+
+try:
+    import torch
+    from torch.nn import CosineEmbeddingLoss, CosineSimilarity, Linear, MSELoss, ReLU, Sequential
+except ImportError:
+    raise ImportError(
+        "Please install Machine Learning dependencies by running 'pip install geniml[ml]'"
+    )
 from huggingface_hub import hf_hub_download
-from torch.nn import CosineEmbeddingLoss, CosineSimilarity, Linear, MSELoss, ReLU, Sequential
 from yaml import safe_dump, safe_load
 
-from ..search.backends import HNSWBackend, QdrantBackend
-from .const import *
+from .const import (
+    CONFIG_FILE_NAME,
+    DEFAULT_BATCH_SIZE,
+    DEFAULT_HUGGINGFACE_MODEL_NAME,
+    DEFAULT_LEARNING_RATE,
+    DEFAULT_LOSS_NAME,
+    DEFAULT_MARGIN,
+    DEFAULT_MUST_TRAINED,
+    DEFAULT_NUM_EPOCHS,
+    DEFAULT_NUM_UNITS,
+    DEFAULT_OPTIMIZER_NAME,
+    DEFAULT_PATIENCE,
+    DEFAULT_PLOT_FILE_NAME,
+    DEFAULT_PLOT_TITLE,
+    MODULE_NAME,
+    TORCH_MODEL_FILE_NAME_PATTERN,
+)
 from .utils import arrays_to_torch_dataloader, dtype_check
 
 _LOGGER = logging.getLogger(MODULE_NAME)
@@ -24,23 +44,26 @@ class Vec2Vec(Sequential):
         input_dim: int,
         output_dim: int,
         num_units: Union[int, List[int]],
-        num_extra_hidden_layers: int,
     ):
         if not isinstance(num_units, list):
-            num_units = [num_units] * (1 + num_extra_hidden_layers)
-        # check if number of layers match length of num_units
-        if len(num_units) != 1 + num_extra_hidden_layers:
-            raise ValueError("list of units number does not match number of layers")
-        # input and first hiden layer
+            num_units = [num_units]
+        num_extra_hidden_layers = len(num_units) - 1
+        # input and first hidden layer
         current_layer_units_num = num_units[0]
-        layers_list = [Linear(in_features=input_dim, out_features=current_layer_units_num), ReLU()]
+        layers_list = [
+            Linear(in_features=input_dim, out_features=current_layer_units_num),
+            ReLU(),
+        ]
         previous_layer_units_num = current_layer_units_num
 
         # extra hidden layer
         for i in range(num_extra_hidden_layers):
             current_layer_units_num = num_units[i + 1]
             layers_list.append(
-                Linear(in_features=previous_layer_units_num, out_features=current_layer_units_num)
+                Linear(
+                    in_features=previous_layer_units_num,
+                    out_features=current_layer_units_num,
+                )
             )
             layers_list.append(ReLU())
             previous_layer_units_num = current_layer_units_num
@@ -54,7 +77,7 @@ class Vec2Vec(Sequential):
 class Vec2VecFNN:
     def __init__(self, model_path: Union[str, None] = None):
         """
-        Initializate Vec2VecFNNtorch.
+        Initialize Vec2VecFNNtorch.
 
         :param model_path: path to the pretrained model on huggingface.
         """
@@ -114,7 +137,6 @@ class Vec2VecFNN:
             config["input_dim"],
             config["output_dim"],
             config["num_units"],
-            config["num_extra_hidden_layers"],
         )
 
         # load the Sequential model from saved files
@@ -159,13 +181,20 @@ class Vec2VecFNN:
         # pytorch tensor's default dtype is float 32
         return self.model(torch.from_numpy(dtype_check(input_vecs))).detach().numpy()
 
-    def compile(self, optimizer: str, loss: str, learning_rate: float):
+    def compile(
+        self,
+        optimizer: str,
+        loss: str,
+        learning_rate: float,
+        margin: Union[float, None] = DEFAULT_MARGIN,
+    ):
         """
-        Configures the model for training.
+        Configure the model for training. This includes setting the optimizer and loss function.
 
         :param optimizer: the name of optimizer
         :param loss: the name of loss function
         :param learning_rate: the learning rate of model backpropagation
+        :param margin: should be a number from −1−1 to 1, 0 to 0.5 is suggested, only for CosineEmbeddingLoss
         """
 
         # set optimizer
@@ -180,7 +209,7 @@ class Vec2VecFNN:
 
         # set loss function
         if loss == "cosine_embedding_loss":
-            self.loss_fn = CosineEmbeddingLoss()
+            self.loss_fn = CosineEmbeddingLoss(margin=margin)
         elif loss == "cosine_similarity":
             self.loss_fn = CosineSimilarity()
         elif loss == "mean_squared_error":
@@ -207,6 +236,8 @@ class Vec2VecFNN:
         num_epochs: int = DEFAULT_NUM_EPOCHS,
         batch_size: int = DEFAULT_BATCH_SIZE,
         learning_rate: float = DEFAULT_LEARNING_RATE,
+        training_target: Union[np.ndarray, None] = None,
+        validating_target: Union[np.ndarray, None] = None,
         **kwargs,
     ):
         """
@@ -219,13 +250,15 @@ class Vec2VecFNN:
         :param save_best: whether the best performance model is saved after each epoch (based on validation loss)
         :param folder_path: the path to the folder to save the model and config
         :param best_model_file_name: the name of the file of saved best model
-        :param early_stop: whether the training should be stoped early to prevent overfitting
+        :param early_stop: whether the training should be stopped early to prevent overfitting
         :param patience: the percentage of epoches to stop training if no validation loss improvement
         :param opt_name: name of optimizer
         :param loss_func: name of loss function
         :param num_epochs: number of training epoches
         :param batch_size: size of batch for training
         :param learning_rate: learning rate of optimizer
+        :param training_target:
+        :param validating_target:
         :param kwargs: see units and layers in reinit_model()
         """
         # if current model is empty, add layers
@@ -237,22 +270,27 @@ class Vec2VecFNN:
             self.config["input_dim"] = input_dim
             self.config["output_dim"] = output_dim
             self.config["num_units"] = kwargs.get("num_units") or DEFAULT_NUM_UNITS
-            self.config["num_extra_hidden_layers"] = (
-                kwargs.get("num_extra_hidden_layers") or DEFAULT_NUM_EXTRA_HIDDEN_LAYERS
-            )
             self.model = Vec2Vec(
                 input_dim=input_dim,
                 output_dim=output_dim,
                 num_units=self.config["num_units"],
-                num_extra_hidden_layers=self.config["num_extra_hidden_layers"],
             )
 
+        if training_target is None:
+            training_target = np.repeat(1, training_X.shape[0])
         # raise the error if validating data is needed but not provided
         if validating_data is not None:
             validating_X, validating_Y = validating_data
+            if validating_target is None:
+                validating_target = np.repeat(1, validating_X.shape[0])
             validating_data = arrays_to_torch_dataloader(
-                validating_X, validating_Y, batch_size=batch_size, shuffle=False
+                validating_X,
+                validating_Y,
+                validating_target,
+                batch_size=batch_size,
+                shuffle=False,
             )
+
             self.most_recent_train["val_loss"] = []
         elif save_best or early_stop:
             raise ValueError("Validating data is not provided")
@@ -265,7 +303,9 @@ class Vec2VecFNN:
         self.compile(optimizer=opt_name, loss=loss_func, learning_rate=learning_rate)
 
         # convert training data from np.ndarray to DataLoader
-        training_data = arrays_to_torch_dataloader(training_X, training_Y, batch_size)
+        training_data = arrays_to_torch_dataloader(
+            training_X, training_Y, training_target, batch_size
+        )
 
         best_val_loss = 1_000_000.0
         patience_count = 0
@@ -284,9 +324,9 @@ class Vec2VecFNN:
                 running_val_loss = 0.0
                 # disable gradient computation
                 with torch.no_grad():
-                    for i, (val_x, val_y) in enumerate(validating_data):
+                    for i, (val_x, val_y, val_target) in enumerate(validating_data):
                         val_output = self.model(val_x)
-                        val_loss = self.calc_loss(val_output, val_y)
+                        val_loss = self.calc_loss(val_output, val_y, val_target)
                         running_val_loss += val_loss
 
                 avg_val_loss = running_val_loss / (i + 1)
@@ -331,7 +371,7 @@ class Vec2VecFNN:
         epoch_loss = 0.0
 
         # train on each batch
-        for i, (x, y) in enumerate(training_data):
+        for i, (x, y, target) in enumerate(training_data):
             # zero gradients for every batch
             self.optimizer.zero_grad()
 
@@ -339,7 +379,7 @@ class Vec2VecFNN:
             outputs = self.model(x)
 
             # compute loss and gradients
-            batch_loss = self.calc_loss(outputs, y)
+            batch_loss = self.calc_loss(outputs, y, target)
             batch_loss.backward()
 
             # adjust learning weights
@@ -349,13 +389,17 @@ class Vec2VecFNN:
             epoch_loss += batch_loss.item()
         return epoch_loss / (i + 1)
 
-    def calc_loss(self, outputs: torch.Tensor, y: torch.Tensor, **kwargs) -> torch.Tensor:
+    def calc_loss(
+        self, outputs: torch.Tensor, y: torch.Tensor, target: torch.Tensor
+    ) -> torch.Tensor:
         """
-        Calculating loss when different loss funcion is given
+        Calculating loss when different loss function is given
 
         :param outputs: the output of model
         :param y: the correct label
+        :param target:
         :return: the loss
+
         """
 
         if not self.config["loss"]:
@@ -365,7 +409,6 @@ class Vec2VecFNN:
         # loss = 1 - cos(output, y)
         # https://pytorch.org/docs/stable/generated/torch.nn.CosineEmbeddingLoss.html
         elif self.config["loss"] == "cosine_embedding_loss":
-            target = kwargs.get("target") or torch.tensor(1.0).repeat(len(y))
             return self.loss_fn(outputs, y, target)
         else:
             return self.loss_fn(outputs, y)
@@ -375,131 +418,32 @@ class Vec2VecFNN:
         save_path: Union[str, None] = None,
         plot_file_name: Union[str, None] = DEFAULT_PLOT_FILE_NAME,
         title: Union[str, None] = DEFAULT_PLOT_TITLE,
-    ):
+    ) -> None:
         """
         Plot the training & validating loss of the most recent training
 
         :param save_path: the path of folder where image will be saved
         :param plot_file_name: the file name of the png file
         :param title: the title in the image
-        :return:
+        :return: None
         """
 
         epoch_range = range(1, len(self.most_recent_train["loss"]) + 1)
         train_loss = self.most_recent_train["loss"]
+        plt.figure()
         plt.plot(epoch_range, train_loss, "r", label="Training loss")
-        if self.most_recent_train["val_loss"]:
+        try:
             valid_loss = self.most_recent_train["val_loss"]
             plt.plot(epoch_range, valid_loss, "b", label="Validation loss")
+        except:
+            pass
         plt.title(title)
         plt.legend()
         if save_path:
             plt.savefig(os.path.join(save_path, plot_file_name))
         else:
             plt.show()
+        plt.close()
 
     def __repr__(self):
         return f"Vec2Vec(input_dimension={self.config['input_dim']}, output_dimension={self.config['output_dim']}, trained={self.trained})"
-
-
-class Text2BEDSearchInterface(object):
-    """
-    search backend interface
-    """
-
-    def __init__(
-        self,
-        nl2vec_model: Union[FlagEmbedding, None],
-        vec2vec_model: Union[Vec2VecFNN, str, None],
-        search_backend: Union[QdrantBackend, HNSWBackend, None],
-    ):
-        """
-        initiate the search interface
-
-        :param nl2vec_model: model that embed natural language to vectors
-        :param vec2vec_model: model that map natural language embedding vectors to region set embedding vectors
-        :param search_backend: search backend that can store vectors and perform KNN search
-        """
-        # load the natural language encoder model
-        if isinstance(nl2vec_model, type(None)):
-            # default FlagEmbedding
-            self.set_flagembedding()
-        else:
-            self.nl2vec = nl2vec_model
-
-        # load the vec2vec model
-        if isinstance(vec2vec_model, Vec2VecFNN):
-            self.vec2vec = vec2vec_model
-        elif isinstance(vec2vec_model, str):
-            self.set_vec2vec(vec2vec_model)
-
-        # init search backend
-        if isinstance(search_backend, type(None)):
-            # init a default HNSWBackend if input is None
-            self.search_backend = HNSWBackend()
-        else:
-            self.search_backend = search_backend
-
-    def set_vec2vec(self, model_name: str):
-        """
-        With a given model_path or huggingface repo, set the vec2vec model
-
-        :param model_name: the path where the model file is saved, or the hugging face repo
-        """
-        self.vec2vec = Vec2VecFNN(model_name)
-
-    def set_flagembedding(self, model_repo: str = DEFAULT_NL_EMBEDDING_MODEL):
-        """
-        With a given huggingface repo, set the nl2vec model as a FlagEmbedding
-
-        :param model_repo: the model repository
-        see https://qdrant.github.io/fastembed/examples/Supported_Models/
-        :return:
-        """
-        _LOGGER.info(f"Setting sentence transformer model {model_repo}")
-        self.nl2vec = FlagEmbedding(model_name=model_repo)
-
-    def nl_vec_search(
-        self,
-        query: Union[str, np.ndarray],
-        limit: int = 10,
-        # offset: int = 0,
-        with_payload: bool = True,
-        with_vectors: bool = False,
-        **kwargs,
-    ) -> List[Dict[str, Union[int, float, Dict[str, str], List[float]]]]:
-        """
-        Given an input natural language, suggest region sets
-
-        :param query: searching input string
-        :param limit: number of results (nearst neighbor in vectors)
-        :param offset: the offset of the search results
-        :param with_payload: whether payloads of vectors in database will be returned [Default: True]
-        :param with_vectors: whether vectors in database will be returned [Default: False]
-        :return: a list of dictionary that contains the search results in this format:
-            {
-                "id": <id>
-                "score": <score>
-                "payload": {
-                    <information of the vector>
-                }
-                "vector": [<the vector>]
-            }
-        """
-
-        # first, get the embedding of the query string
-        if isinstance(query, str):
-            query = next(self.nl2vec.embed(query))
-        search_vector = self.vec2vec.embedding_to_embedding(query)
-        # perform the KNN search among vectors stored in backend
-        return self.search_backend.search(
-            search_vector,
-            limit,
-            with_payload=with_payload,
-            with_vectors=with_vectors,
-            # offset=offset)
-            **kwargs,
-        )
-
-    def __repr__(self):
-        return f"Text2BEDSearchInterface(nl2vec_model={self.nl2vec}, vec2vec_model={self.vec2vec}, search_backend={self.search_backend})"
